@@ -1,4 +1,5 @@
 from __future__ import annotations
+import numpy as np
 from typing import Dict, List, Optional, Type, Union
 
 from autoconf import cached_property
@@ -23,6 +24,8 @@ class AbstractToInversion:
         dataset: Optional[Union[aa.Imaging, aa.Interferometer, aa.DatasetInterface]],
         adapt_images: Optional[AdaptImages] = None,
         settings_inversion: aa.SettingsInversion = aa.SettingsInversion(),
+        preloads: aa.Preloads = None,
+        xp=np,
     ):
         """
         Abstract class which interfaces a dataset and input modeling object (e.g. galaxies, a tracer) with the
@@ -75,6 +78,10 @@ class AbstractToInversion:
 
         self.settings_inversion = settings_inversion
 
+        self.preloads = preloads
+
+        self._xp = xp
+
     @property
     def psf(self) -> Optional[aa.Kernel2D]:
         """
@@ -188,6 +195,8 @@ class GalaxiesToInversion(AbstractToInversion):
         galaxies: List[Galaxy],
         adapt_images: Optional[AdaptImages] = None,
         settings_inversion: aa.SettingsInversion = aa.SettingsInversion(),
+        preloads: aa.Preloads = None,
+        xp=np,
     ):
         """
         Interfaces a dataset and input list of galaxies with the inversion module. to setup a
@@ -229,6 +238,8 @@ class GalaxiesToInversion(AbstractToInversion):
             dataset=dataset,
             adapt_images=adapt_images,
             settings_inversion=settings_inversion,
+            preloads=preloads,
+            xp=xp,
         )
 
     @property
@@ -304,6 +315,7 @@ class GalaxiesToInversion(AbstractToInversion):
                             psf=self.dataset.psf,
                             light_profile_list=light_profile_list,
                             regularization=light_profile.regularization,
+                            xp=self._xp,
                         )
 
                         lp_linear_func_galaxy_dict[lp_linear_func] = galaxy
@@ -390,34 +402,11 @@ class GalaxiesToInversion(AbstractToInversion):
         image_plane_mesh_grid_list = []
 
         for galaxy in self.galaxies.galaxies_with_cls_list_from(cls=aa.Pixelization):
-            pixelization = galaxy.cls_list_from(cls=aa.Pixelization)[0]
-
-            if pixelization.image_mesh is not None:
-                try:
-                    adapt_data = self.adapt_images.galaxy_image_dict[galaxy]
-                except (AttributeError, KeyError):
-                    adapt_data = None
-
-                    if pixelization.image_mesh.uses_adapt_images:
-                        raise aa.exc.PixelizationException(
-                            """
-                            Attempted to perform fit using a pixelization which requires an 
-                            image-mesh (E.g. KMeans, Hilbert).
-                            
-                            However, the adapt-images passed to the fit (E.g. FitImaging, FitInterferometer) 
-                            is None. Without an adapt image, an image-mesh cannot be used.
-                            """
-                        )
-
+            try:
                 image_plane_mesh_grid = (
-                    pixelization.image_mesh.image_plane_mesh_grid_from(
-                        mask=self.dataset.mask,
-                        adapt_data=adapt_data,
-                        settings=self.settings_inversion,
-                    )
+                    self.adapt_images.galaxy_image_plane_mesh_grid_dict[galaxy]
                 )
-
-            else:
+            except (AttributeError, KeyError, TypeError):
                 image_plane_mesh_grid = None
 
             image_plane_mesh_grid_list.append(image_plane_mesh_grid)
@@ -482,11 +471,15 @@ class GalaxiesToInversion(AbstractToInversion):
             source_plane_mesh_grid=source_plane_mesh_grid,
             image_plane_mesh_grid=image_plane_mesh_grid,
             adapt_data=adapt_galaxy_image,
+            preloads=self.preloads,
+            xp=self._xp,
         )
 
         return mapper_from(
             mapper_grids=mapper_grids,
             regularization=regularization,
+            preloads=self.preloads,
+            xp=self._xp,
         )
 
     @cached_property
@@ -568,6 +561,8 @@ class GalaxiesToInversion(AbstractToInversion):
             dataset=self.dataset,
             linear_obj_list=self.linear_obj_list,
             settings=self.settings_inversion,
+            preloads=self.preloads,
+            xp=self._xp,
         )
 
         inversion.linear_obj_galaxy_dict = self.linear_obj_galaxy_dict
