@@ -1,10 +1,9 @@
-import jax
-from jax import jit
-import jax.numpy as jnp
-from functools import wraps, partial
+from functools import wraps
 import logging
+import numpy as np
 from typing import List, Tuple, Union
 
+from autoconf import conf
 
 import autoarray as aa
 
@@ -46,18 +45,49 @@ def precompute_jacobian(func):
     return wrapper
 
 
-def one_step(r, _, theta, fun, fun_dr):
-    r = jnp.abs(r - fun(r, theta) / fun_dr(r, theta))
-    return r, None
+def evaluation_grid(func):
+    @wraps(func)
+    def wrapper(
+        lensing_obj, grid, pixel_scale: Union[Tuple[float, float], float] = 0.05
+    ):
+        if hasattr(grid, "is_evaluation_grid"):
+            if grid.is_evaluation_grid:
+                return func(lensing_obj, grid, pixel_scale)
 
+        pixel_scale_ratio = grid.pixel_scale / pixel_scale
 
-@partial(jit, static_argnums=(4,))
-def step_r(r, theta, fun, fun_dr, N=20):
-    one_step_partial = jax.tree_util.Partial(
-        one_step, theta=theta, fun=fun, fun_dr=fun_dr
-    )
-    new_r = jax.lax.scan(one_step_partial, r, xs=jnp.arange(N))[0]
-    return jnp.stack([new_r * jnp.sin(theta), new_r * jnp.cos(theta)]).T
+        zoom = aa.Zoom2D(mask=grid.mask)
+
+        zoom_shape_native = zoom.shape_native
+        shape_native = (
+            int(pixel_scale_ratio * zoom_shape_native[0]),
+            int(pixel_scale_ratio * zoom_shape_native[1]),
+        )
+
+        max_evaluation_grid_size = conf.instance["general"]["grid"][
+            "max_evaluation_grid_size"
+        ]
+
+        # This is a hack to prevent the evaluation gird going beyond 1000 x 1000 pixels, which slows the code
+        # down a lot. Need a better moe robust way to set this up for any general lens.
+
+        if shape_native[0] > max_evaluation_grid_size:
+            pixel_scale = pixel_scale_ratio / (
+                shape_native[0] / float(max_evaluation_grid_size)
+            )
+            shape_native = (max_evaluation_grid_size, max_evaluation_grid_size)
+
+        grid = aa.Grid2D.uniform(
+            shape_native=shape_native,
+            pixel_scales=(pixel_scale, pixel_scale),
+            origin=zoom.offset_scaled,
+        )
+
+        grid.is_evaluation_grid = True
+
+        return func(lensing_obj, grid, pixel_scale)
+
+    return wrapper
 
 
 class OperateDeflections:
@@ -79,26 +109,10 @@ class OperateDeflections:
     def deflections_yx_2d_from(self, grid: aa.type.Grid2DLike, **kwargs):
         raise NotImplementedError
 
-    def deflections_yx_scalar(self, y, x, pixel_scales):
-
-        # A version of the deflection function that takes in two scalars
-        # and outputs a 2D vector.  Needed for JAX auto differentiation.
-
-        mask = aa.Mask2D.all_false(
-            shape_native=(1, 1),
-            pixel_scales=pixel_scales,
-        )
-
-        g = aa.Grid2D(
-            values=jnp.stack((y.reshape(1), x.reshape(1)), axis=-1), mask=mask
-        )
-
-        return self.deflections_yx_2d_from(g).squeeze()
-
     def __eq__(self, other):
         return self.__dict__ == other.__dict__ and self.__class__ is other.__class__
 
-    def time_delay_geometry_term_from(self, grid) -> aa.Array2D:
+    def time_delay_geometry_term_from(self, grid, xp=np) -> aa.Array2D:
         """
             Returns the geometric time delay term of the Fermat potential for a given grid of image-plane positions.
 
@@ -122,7 +136,7 @@ class OperateDeflections:
             -------
             The geometric time delay term at each grid position.
         """
-        deflections = self.deflections_yx_2d_from(grid=grid)
+        deflections = self.deflections_yx_2d_from(grid=grid, xp=xp)
 
         src_y = grid[:, 0] - deflections[:, 0]
         src_x = grid[:, 1] - deflections[:, 1]
@@ -133,7 +147,7 @@ class OperateDeflections:
             return aa.ArrayIrregular(values=delay)
         return aa.Array2D(values=delay, mask=grid.mask)
 
-    def fermat_potential_from(self, grid) -> aa.Array2D:
+    def fermat_potential_from(self, grid, xp=np) -> aa.Array2D:
         """
         Returns the Fermat potential for a given grid of image-plane positions.
 
@@ -158,8 +172,8 @@ class OperateDeflections:
         -------
         The Fermat potential at each grid position.
         """
-        time_delay_geometry_term = self.time_delay_geometry_term_from(grid=grid)
-        potential = self.potential_2d_from(grid=grid)
+        time_delay_geometry_term = self.time_delay_geometry_term_from(grid=grid, xp=xp)
+        potential = self.potential_2d_from(grid=grid, xp=xp)
 
         fermat_potential = time_delay_geometry_term - potential
 
@@ -167,27 +181,6 @@ class OperateDeflections:
             return aa.ArrayIrregular(values=fermat_potential)
         return aa.Array2D(values=fermat_potential, mask=grid.mask)
 
-    def time_delays_from(self, grid) -> aa.Array2D:
-        """
-        Returns the 2D time delay map of lensing object, which is computed as the deflection angles in the y and x
-        directions multiplied by the y and x coordinates of the grid.
-
-        Parameters
-        ----------
-        grid
-            The 2D grid of (y,x) arc-second coordinates the deflection angles and time delay are computed on.
-        """
-        deflections_yx = self.deflections_yx_2d_from(grid=grid)
-
-        return aa.Array2D(
-            values=deflections_yx[:, 0] * grid[:, 0]
-            + deflections_yx[:, 1] * grid[:, 1],
-            mask=grid.mask,
-        )
-
-    def __hash__(self):
-        return hash(repr(self))
-
     @precompute_jacobian
     def tangential_eigen_value_from(self, grid, jacobian=None) -> aa.Array2D:
         """
@@ -249,7 +242,9 @@ class OperateDeflections:
 
         return aa.Array2D(values=1 / det_jacobian, mask=grid.mask)
 
-    def hessian_from(self, grid, buffer: float = 0.01, deflections_func=None) -> Tuple:
+    def hessian_from(
+        self, grid, buffer: float = 0.01, deflections_func=None, xp=np
+    ) -> Tuple:
         """
         Returns the Hessian of the lensing object, where the Hessian is the second partial derivatives of the
         potential (see equation 55 https://inspirehep.net/literature/419263):
@@ -277,29 +272,26 @@ class OperateDeflections:
         if deflections_func is None:
             deflections_func = self.deflections_yx_2d_from
 
-        if isinstance(grid, aa.Grid2DIrregular):
-            grid = grid.array
-
         grid_shift_y_up = aa.Grid2DIrregular(
-            values=jnp.stack([grid[:, 0] + buffer, grid[:, 1]], axis=1)
+            values=xp.stack([grid[:, 0] + buffer, grid[:, 1]], axis=1)
         )
 
         grid_shift_y_down = aa.Grid2DIrregular(
-            values=jnp.stack([grid[:, 0] - buffer, grid[:, 1]], axis=1)
+            values=xp.stack([grid[:, 0] - buffer, grid[:, 1]], axis=1)
         )
 
         grid_shift_x_left = aa.Grid2DIrregular(
-            values=jnp.stack([grid[:, 0], grid[:, 1] - buffer], axis=1)
+            values=xp.stack([grid[:, 0], grid[:, 1] - buffer], axis=1)
         )
 
         grid_shift_x_right = aa.Grid2DIrregular(
-            values=jnp.stack([grid[:, 0], grid[:, 1] + buffer], axis=1)
+            values=xp.stack([grid[:, 0], grid[:, 1] + buffer], axis=1)
         )
 
-        deflections_up = deflections_func(grid=grid_shift_y_up)
-        deflections_down = deflections_func(grid=grid_shift_y_down)
-        deflections_left = deflections_func(grid=grid_shift_x_left)
-        deflections_right = deflections_func(grid=grid_shift_x_right)
+        deflections_up = deflections_func(grid=grid_shift_y_up, xp=xp)
+        deflections_down = deflections_func(grid=grid_shift_y_down, xp=xp)
+        deflections_left = deflections_func(grid=grid_shift_x_left, xp=xp)
+        deflections_right = deflections_func(grid=grid_shift_x_right, xp=xp)
 
         hessian_yy = 0.5 * (deflections_up[:, 0] - deflections_down[:, 0]) / buffer
         hessian_xy = 0.5 * (deflections_up[:, 1] - deflections_down[:, 1]) / buffer
@@ -375,12 +367,15 @@ class OperateDeflections:
         gamma_1 = 0.5 * (hessian_xx - hessian_yy)
         gamma_2 = hessian_xy
 
-        shear_yx_2d = jnp.stack([gamma_2.array, gamma_1.array], axis=1)
+        shear_yx_2d = np.zeros(shape=(grid.shape_slim, 2))
+
+        shear_yx_2d[:, 0] = gamma_2
+        shear_yx_2d[:, 1] = gamma_1
 
         return ShearYX2DIrregular(values=shear_yx_2d, grid=grid)
 
     def magnification_2d_via_hessian_from(
-        self, grid, buffer: float = 0.01, deflections_func=None
+        self, grid, buffer: float = 0.01, deflections_func=None, xp=np
     ) -> aa.ArrayIrregular:
         """
         Returns the 2D magnification map of lensing object, which is computed from the 2D deflection angle map
@@ -402,7 +397,7 @@ class OperateDeflections:
             The 2D grid of (y,x) arc-second coordinates the deflection angles and magnification map are computed on.
         """
         hessian_yy, hessian_xy, hessian_yx, hessian_xx = self.hessian_from(
-            grid=grid, buffer=buffer, deflections_func=deflections_func
+            grid=grid, buffer=buffer, deflections_func=deflections_func, xp=xp
         )
 
         det_A = (1 - hessian_xx) * (1 - hessian_yy) - hessian_xy * hessian_yx
@@ -419,9 +414,9 @@ class OperateDeflections:
 
         return grid_contour.contour_list
 
+    @evaluation_grid
     def tangential_critical_curve_list_from(
-        self,
-        grid,
+        self, grid, pixel_scale: Union[Tuple[float, float], float] = 0.05
     ) -> List[aa.Grid2DIrregular]:
         """
         Returns all tangential critical curves of the lensing system, which are computed as follows:
@@ -445,9 +440,9 @@ class OperateDeflections:
 
         return self.contour_list_from(grid=grid, contour_array=tangential_eigen_values)
 
+    @evaluation_grid
     def radial_critical_curve_list_from(
-        self,
-        grid,
+        self, grid, pixel_scale: Union[Tuple[float, float], float] = 0.05
     ) -> List[aa.Grid2DIrregular]:
         """
         Returns all radial critical curves of the lensing system, which are computed as follows:
@@ -471,9 +466,9 @@ class OperateDeflections:
 
         return self.contour_list_from(grid=grid, contour_array=radial_eigen_values)
 
+    @evaluation_grid
     def tangential_caustic_list_from(
-        self,
-        grid,
+        self, grid, pixel_scale: Union[Tuple[float, float], float] = 0.05
     ) -> List[aa.Grid2DIrregular]:
         """
         Returns all tangential caustics of the lensing system, which are computed as follows:
@@ -497,7 +492,7 @@ class OperateDeflections:
         """
 
         tangential_critical_curve_list = self.tangential_critical_curve_list_from(
-            grid=grid
+            grid=grid, pixel_scale=pixel_scale
         )
 
         tangential_caustic_list = []
@@ -513,9 +508,9 @@ class OperateDeflections:
 
         return tangential_caustic_list
 
+    @evaluation_grid
     def radial_caustic_list_from(
-        self,
-        grid,
+        self, grid, pixel_scale: Union[Tuple[float, float], float] = 0.05
     ) -> List[aa.Grid2DIrregular]:
         """
         Returns all radial caustics of the lensing system, which are computed as follows:
@@ -538,7 +533,9 @@ class OperateDeflections:
             caustic to be computed more accurately using a higher resolution grid.
         """
 
-        radial_critical_curve_list = self.radial_critical_curve_list_from(grid=grid)
+        radial_critical_curve_list = self.radial_critical_curve_list_from(
+            grid=grid, pixel_scale=pixel_scale
+        )
 
         radial_caustic_list = []
 
@@ -553,7 +550,10 @@ class OperateDeflections:
 
         return radial_caustic_list
 
-    def radial_critical_curve_area_list_from(self, grid) -> List[float]:
+    @evaluation_grid
+    def radial_critical_curve_area_list_from(
+        self, grid, pixel_scale: Union[Tuple[float, float], float]
+    ) -> List[float]:
         """
         Returns the surface area within each radial critical curve as a list, the calculation of which is described in
         the function `radial_critical_curve_list_from()`.
@@ -573,13 +573,15 @@ class OperateDeflections:
             If input, the `evaluation_grid` decorator creates the 2D grid at this resolution, therefore enabling the
             caustic to be computed more accurately using a higher resolution grid.
         """
-        radial_critical_curve_list = self.radial_critical_curve_list_from(grid=grid)
+        radial_critical_curve_list = self.radial_critical_curve_list_from(
+            grid=grid, pixel_scale=pixel_scale
+        )
 
         return self.area_within_curve_list_from(curve_list=radial_critical_curve_list)
 
+    @evaluation_grid
     def tangential_critical_curve_area_list_from(
-        self,
-        grid,
+        self, grid, pixel_scale: Union[Tuple[float, float], float] = 0.05
     ) -> List[float]:
         """
         Returns the surface area within each tangential critical curve as a list, the calculation of which is
@@ -600,7 +602,7 @@ class OperateDeflections:
             caustic to be computed more accurately using a higher resolution grid.
         """
         tangential_critical_curve_list = self.tangential_critical_curve_list_from(
-            grid=grid
+            grid=grid, pixel_scale=pixel_scale
         )
 
         return self.area_within_curve_list_from(
@@ -614,14 +616,14 @@ class OperateDeflections:
 
         for curve in curve_list:
             x, y = curve[:, 0], curve[:, 1]
-            area = jnp.abs(0.5 * jnp.sum(y[:-1] * jnp.diff(x) - x[:-1] * jnp.diff(y)))
+            area = np.abs(0.5 * np.sum(y[:-1] * np.diff(x) - x[:-1] * np.diff(y)))
             area_within_each_curve_list.append(area)
 
         return area_within_each_curve_list
 
+    @evaluation_grid
     def einstein_radius_list_from(
-        self,
-        grid,
+        self, grid, pixel_scale: Union[Tuple[float, float], float] = 0.05
     ):
         """
         Returns a list of the Einstein radii corresponding to the area within each tangential critical curve.
@@ -648,14 +650,16 @@ class OperateDeflections:
             caustic to be computed more accurately using a higher resolution grid.
         """
         try:
-            area_list = self.tangential_critical_curve_area_list_from(grid=grid)
-            return [jnp.sqrt(area / jnp.pi) for area in area_list]
+            area_list = self.tangential_critical_curve_area_list_from(
+                grid=grid, pixel_scale=pixel_scale
+            )
+            return [np.sqrt(area / np.pi) for area in area_list]
         except TypeError:
             raise TypeError("The grid input was unable to estimate the Einstein Radius")
 
+    @evaluation_grid
     def einstein_radius_from(
-        self,
-        grid,
+        self, grid, pixel_scale: Union[Tuple[float, float], float] = 0.05
     ):
         """
         Returns the Einstein radius corresponding to the area within the tangential critical curve.
@@ -697,9 +701,9 @@ class OperateDeflections:
 
         return sum(einstein_radii_list)
 
+    @evaluation_grid
     def einstein_mass_angular_list_from(
-        self,
-        grid,
+        self, grid, pixel_scale: Union[Tuple[float, float], float] = 0.05
     ) -> List[float]:
         """
         Returns a list of the angular Einstein massses corresponding to the area within each tangential critical curve.
@@ -728,12 +732,14 @@ class OperateDeflections:
             If input, the `evaluation_grid` decorator creates the 2D grid at this resolution, therefore enabling the
             caustic to be computed more accurately using a higher resolution grid.
         """
-        einstein_radius_list = self.einstein_radius_list_from(grid=grid)
-        return [jnp.pi * einstein_radius**2 for einstein_radius in einstein_radius_list]
+        einstein_radius_list = self.einstein_radius_list_from(
+            grid=grid, pixel_scale=pixel_scale
+        )
+        return [np.pi * einstein_radius**2 for einstein_radius in einstein_radius_list]
 
+    @evaluation_grid
     def einstein_mass_angular_from(
-        self,
-        grid,
+        self, grid, pixel_scale: Union[Tuple[float, float], float] = 0.05
     ) -> float:
         """
         Returns the Einstein radius corresponding to the area within the tangential critical curve.
@@ -762,7 +768,9 @@ class OperateDeflections:
             If input, the `evaluation_grid` decorator creates the 2D grid at this resolution, therefore enabling the
             caustic to be computed more accurately using a higher resolution grid.
         """
-        einstein_mass_angular_list = self.einstein_mass_angular_list_from(grid=grid)
+        einstein_mass_angular_list = self.einstein_mass_angular_list_from(
+            grid=grid, pixel_scale=pixel_scale
+        )
 
         if len(einstein_mass_angular_list) > 1:
             logger.info(
@@ -774,167 +782,6 @@ class OperateDeflections:
 
         return einstein_mass_angular_list[0]
 
-    def jacobian_stack(self, y, x, pixel_scales):
-        return jnp.stack(
-            jax.jacfwd(self.deflections_yx_scalar, argnums=(0, 1))(y, x, pixel_scales)
-        )
-
-    def jacobian_stack_vector(self, y, x, pixel_scales):
-        return jnp.vectorize(
-            jax.tree_util.Partial(self.jacobian_stack, pixel_scales=pixel_scales),
-            signature="(),()->(i,i)",
-        )(y, x)
-
-    def convergence_mag_shear_yx(self, y, x):
-        J = self.jacobian_stack_vector(y, x, 0.05)
-        K = 0.5 * (J[..., 0, 0] + J[..., 1, 1])
-        mag_shear = 0.5 * jnp.sqrt(
-            (J[..., 0, 1] + J[..., 1, 0]) ** 2 + (J[..., 0, 0] - J[..., 1, 1]) ** 2
-        )
-        return K, mag_shear
-
-    @partial(jit, static_argnums=(0,))
-    def tangential_eigen_value_yx(self, y, x):
-        K, mag_shear = self.convergence_mag_shear_yx(y, x)
-        return 1 - K - mag_shear
-
-    @partial(jit, static_argnums=(0, 3))
-    def tangential_eigen_value_rt(self, r, theta, centre=(0.0, 0.0)):
-        y = r * jnp.sin(theta) + centre[0]
-        x = r * jnp.cos(theta) + centre[1]
-        return self.tangential_eigen_value_yx(y, x)
-
-    @partial(jit, static_argnums=(0, 3))
-    def grad_r_tangential_eigen_value(self, r, theta, centre=(0.0, 0.0)):
-        # ignore `self` with the `argnums` below
-        tangential_eigen_part = partial(self.tangential_eigen_value_rt, centre=centre)
-        return jnp.vectorize(
-            jax.jacfwd(tangential_eigen_part, argnums=(0,)), signature="(),()->()"
-        )(r, theta)[0]
-
-    @partial(jit, static_argnums=(0,))
-    def radial_eigen_value_yx(self, y, x):
-        K, mag_shear = self.convergence_mag_shear_yx(y, x)
-        return 1 - K + mag_shear
-
-    @partial(jit, static_argnums=(0, 3))
-    def radial_eigen_value_rt(self, r, theta, centre=(0.0, 0.0)):
-        y = r * jnp.sin(theta) + centre[0]
-        x = r * jnp.cos(theta) + centre[1]
-        return self.radial_eigen_value_yx(y, x)
-
-    @partial(jit, static_argnums=(0, 3))
-    def grad_r_radial_eigen_value(self, r, theta, centre=(0.0, 0.0)):
-        # ignore `self` with the `argnums` below
-        radial_eigen_part = partial(self.radial_eigen_value_rt, centre=centre)
-        return jnp.vectorize(
-            jax.jacfwd(radial_eigen_part, argnums=(0,)), signature="(),()->()"
-        )(r, theta)[0]
-
-    def tangential_critical_curve_jax(
-        self,
-        init_r=0.1,
-        init_centre=(0.0, 0.0),
-        n_points=300,
-        n_steps=20,
-        threshold=1e-5,
-    ):
-        """
-        Returns all tangential critical curves of the lensing system, which are computed as follows:
-
-        1) Create a set of `n_points` initial points in a circle of radius `init_r` and centred on `init_centre`
-        2) Apply `n_steps` of Newton's method to these points in the "radial" direction only (i.e. keeping angle fixed).
-        Jax's auto differentiation is used to find the radial derivatives of the tangential eigen value function for
-        this step.
-        3) Filter the results and only keep point that have their tangential eigen value `threshold` of 0
-
-        No underlying grid is needed for the method, but the quality of the results are dependent on the initial
-        circle of points.
-
-        Parameters
-        ----------
-        init_r : float
-            Radius of the circle of initial guess points
-        init_centre : tuple
-            centre of the circle of initial guess points as `(y, x)`
-        n_points : Int
-            Number of initial guess points to create (evenly spaced in angle around `init_centre`)
-        n_steps : Int
-            Number of iterations of Newton's method to apply
-        threshold : float
-            Only keep points whose tangential eigen value is within this value of zero (inclusive)
-        """
-        r = jnp.ones(n_points) * init_r
-        theta = jnp.linspace(0, 2 * jnp.pi, n_points + 1)[:-1]
-        new_yx = step_r(
-            r,
-            theta,
-            jax.tree_util.Partial(self.tangential_eigen_value_rt, centre=init_centre),
-            jax.tree_util.Partial(
-                self.grad_r_tangential_eigen_value, centre=init_centre
-            ),
-            n_steps,
-        )
-        new_yx = new_yx + jnp.array(init_centre)
-        # filter out nan values
-        fdx = jnp.isfinite(new_yx).all(axis=1)
-        new_yx = new_yx[fdx]
-        # filter out failed points
-        value = jnp.abs(self.tangential_eigen_value_yx(new_yx[:, 0], new_yx[:, 1]))
-        gdx = value <= threshold
-        return aa.structures.grids.irregular_2d.Grid2DIrregular(values=new_yx[gdx])
-
-    def radial_critical_curve_jax(
-        self,
-        init_r=0.01,
-        init_centre=(0.0, 0.0),
-        n_points=300,
-        n_steps=20,
-        threshold=1e-5,
-    ):
-        """
-        Returns all radial critical curves of the lensing system, which are computed as follows:
-
-        1) Create a set of `n_points` initial points in a circle of radius `init_r` and centred on `init_centre`
-        2) Apply `n_steps` of Newton's method to these points in the "radial" direction only (i.e. keeping angle fixed).
-        Jax's auto differentiation is used to find the radial derivatives of the radial eigen value function for
-        this step.
-        3) Filter the results and only keep point that have their radial eigen value `threshold` of 0
-
-        No underlying grid is needed for the method, but the quality of the results are dependent on the initial
-        circle of points.
-
-        Parameters
-        ----------
-        init_r : float
-            Radius of the circle of initial guess points
-        init_centre : tuple
-            centre of the circle of initial guess points as `(y, x)`
-        n_points : Int
-            Number of initial guess points to create (evenly spaced in angle around `init_centre`)
-        n_steps : Int
-            Number of iterations of Newton's method to apply
-        threshold : float
-            Only keep points whose radial eigen value is within this value of zero (inclusive)
-        """
-        r = jnp.ones(n_points) * init_r
-        theta = jnp.linspace(0, 2 * jnp.pi, n_points + 1)[:-1]
-        new_yx = step_r(
-            r,
-            theta,
-            jax.tree_util.Partial(self.radial_eigen_value_rt, centre=init_centre),
-            jax.tree_util.Partial(self.grad_r_radial_eigen_value, centre=init_centre),
-            n_steps,
-        )
-        new_yx = new_yx + jnp.array(init_centre)
-        # filter out nan values
-        fdx = jnp.isfinite(new_yx).all(axis=1)
-        new_yx = new_yx[fdx]
-        # filter out failed points
-        value = jnp.abs(self.radial_eigen_value_yx(new_yx[:, 0], new_yx[:, 1]))
-        gdx = value <= threshold
-        return aa.structures.grids.irregular_2d.Grid2DIrregular(values=new_yx[gdx])
-
     def jacobian_from(self, grid):
         """
         Returns the Jacobian of the lensing object, which is computed by taking the gradient of the 2D deflection
@@ -952,24 +799,36 @@ class OperateDeflections:
         grid
             The 2D grid of (y,x) arc-second coordinates the deflection angles and Jacobian are computed on.
         """
-        A = self.jacobian_stack_vector(
-            grid.array[:, 0], grid.array[:, 1], grid.pixel_scales
-        )
-        a = jnp.eye(2).reshape(1, 2, 2) - A
-        return [
-            [
-                aa.Array2D(values=a[..., 1, 1], mask=grid.mask),
-                aa.Array2D(values=a[..., 1, 0], mask=grid.mask),
-            ],
-            [
-                aa.Array2D(values=a[..., 0, 1], mask=grid.mask),
-                aa.Array2D(values=a[..., 0, 0], mask=grid.mask),
-            ],
-        ]
 
-        # transpose the result
-        # use `moveaxis` as grid might not be nx2
-        # return jnp.moveaxis(jnp.moveaxis(a, -1, 0), -1, 0)
+        deflections = self.deflections_yx_2d_from(grid=grid)
+
+        # TODO : Can probably make this work on irregular grid? Is there any point?
+
+        a11 = aa.Array2D(
+            values=1.0
+            - np.gradient(deflections.native[:, :, 1], grid.native[0, :, 1], axis=1),
+            mask=grid.mask,
+        )
+
+        a12 = aa.Array2D(
+            values=-1.0
+            * np.gradient(deflections.native[:, :, 1], grid.native[:, 0, 0], axis=0),
+            mask=grid.mask,
+        )
+
+        a21 = aa.Array2D(
+            values=-1.0
+            * np.gradient(deflections.native[:, :, 0], grid.native[0, :, 1], axis=1),
+            mask=grid.mask,
+        )
+
+        a22 = aa.Array2D(
+            values=1
+            - np.gradient(deflections.native[:, :, 0], grid.native[:, 0, 0], axis=0),
+            mask=grid.mask,
+        )
+
+        return [[a11, a12], [a21, a22]]
 
     @precompute_jacobian
     def convergence_2d_via_jacobian_from(self, grid, jacobian=None) -> aa.Array2D:
@@ -1018,9 +877,10 @@ class OperateDeflections:
         jacobian
             A precomputed lensing jacobian, which is passed throughout the `CalcLens` functions for efficiency.
         """
-        shear_y = -0.5 * (jacobian[0][1] + jacobian[1][0]).array
-        shear_x = 0.5 * (jacobian[1][1] - jacobian[0][0]).array
-        shear_yx_2d = jnp.stack([shear_y, shear_x]).T
+
+        shear_yx_2d = np.zeros(shape=(grid.shape_slim, 2))
+        shear_yx_2d[:, 0] = -0.5 * (jacobian[0][1] + jacobian[1][0])
+        shear_yx_2d[:, 1] = 0.5 * (jacobian[1][1] - jacobian[0][0])
 
         if isinstance(grid, aa.Grid2DIrregular):
             return ShearYX2DIrregular(values=shear_yx_2d, grid=grid)
