@@ -1,6 +1,5 @@
 from __future__ import annotations
-import jax
-import jax.numpy as jnp
+import numpy as np
 from typing import TYPE_CHECKING, Dict, List, Optional
 
 from autoarray import Array2D
@@ -23,7 +22,7 @@ class OperateImage:
     """
 
     def image_2d_from(
-        self, grid: aa.Grid2D, operated_only: Optional[bool] = None
+        self, grid: aa.Grid2D, xp=np, operated_only: Optional[bool] = None
     ) -> aa.Array2D:
         raise NotImplementedError
 
@@ -35,11 +34,11 @@ class OperateImage:
         image_2d: aa.Array2D,
         blurring_image_2d: aa.Array2D,
         psf: aa.Kernel2D,
+        xp=np,
     ) -> aa.Array2D:
 
         values = psf.convolved_image_from(
-            image=image_2d,
-            blurring_image=blurring_image_2d,
+            image=image_2d, blurring_image=blurring_image_2d, xp=xp
         )
         return Array2D(values=values, mask=image_2d.mask)
 
@@ -48,6 +47,7 @@ class OperateImage:
         grid: aa.Grid2D,
         blurring_grid: aa.Grid2D,
         psf: aa.Kernel2D = None,
+        xp=np,
     ) -> aa.Array2D:
         """
         Evaluate the light object's 2D image from a input 2D grid of coordinates and convolve it with a PSF.
@@ -72,24 +72,27 @@ class OperateImage:
             LightProfileOperated,
         )
 
-        image_2d_not_operated = self.image_2d_from(grid=grid, operated_only=False)
+        image_2d_not_operated = self.image_2d_from(
+            grid=grid, xp=xp, operated_only=False
+        )
         blurring_image_2d_not_operated = self.image_2d_from(
-            grid=blurring_grid, operated_only=False
+            grid=blurring_grid, xp=xp, operated_only=False
         )
 
         blurred_image_2d = self._blurred_image_2d_from(
             image_2d=image_2d_not_operated,
             blurring_image_2d=blurring_image_2d_not_operated,
             psf=psf,
+            xp=xp,
         )
 
         if self.has(cls=LightProfileOperated):
-            image_2d_operated = self.image_2d_from(grid=grid, operated_only=True)
+            image_2d_operated = self.image_2d_from(grid=grid, xp=xp, operated_only=True)
             return blurred_image_2d + image_2d_operated
 
         return blurred_image_2d
 
-    def padded_image_2d_from(self, grid, psf_shape_2d):
+    def padded_image_2d_from(self, grid, psf_shape_2d, xp=np):
         """
         Evaluate the light object's 2D image from a input 2D grid of padded coordinates, where this padding is
         sufficient to encapsulate all surrounding pixels that will blur light into the original image given the
@@ -111,7 +114,7 @@ class OperateImage:
         """
         padded_grid = grid.padded_grid_from(kernel_shape_native=psf_shape_2d)
 
-        return self.image_2d_from(grid=padded_grid)
+        return self.image_2d_from(grid=padded_grid, xp=xp)
 
     def unmasked_blurred_image_2d_from(self, grid, psf):
         """
@@ -161,7 +164,7 @@ class OperateImage:
         return padded_image_2d + padded_image_2d_operated
 
     def visibilities_from(
-        self, grid: aa.Grid2D, transformer: aa.type.Transformer
+        self, grid: aa.Grid2D, transformer: aa.type.Transformer, xp=np
     ) -> aa.Visibilities:
         """
         Evaluate the light object's 2D image from a input 2D grid of coordinates and transform this to an array of
@@ -190,8 +193,8 @@ class OperateImage:
 
         if self.has(cls=LightProfile) or isinstance(self, LightProfile):
 
-            image_2d = self.image_2d_from(grid=grid)
-            return transformer.visibilities_from(image=image_2d)
+            image_2d = self.image_2d_from(grid=grid, xp=xp)
+            return transformer.visibilities_from(image=image_2d, xp=xp)
 
         return aa.Visibilities.zeros(shape_slim=(transformer.uv_wavelengths.shape[0],))
 
@@ -304,7 +307,10 @@ class OperateImageList(OperateImage):
         return unmasked_blurred_image_list
 
     def visibilities_list_from(
-        self, grid: aa.Grid2D, transformer: aa.type.Transformer
+        self,
+        grid: aa.Grid2D,
+        transformer: aa.type.Transformer,
+        xp=np,
     ) -> List[aa.Array2D]:
         """
         Evaluate the light object's list of 2D image from a input 2D grid of coordinates and transform each image to
@@ -334,12 +340,12 @@ class OperateImageList(OperateImage):
         visibilities_list = []
 
         for image_2d in image_2d_list:
-            if not jnp.any(image_2d.array):
+            if not xp.any(image_2d.array):
                 visibilities = aa.Visibilities.zeros(
                     shape_slim=(transformer.uv_wavelengths.shape[0],)
                 )
             else:
-                visibilities = transformer.visibilities_from(image=image_2d)
+                visibilities = transformer.visibilities_from(image=image_2d, xp=xp)
 
             visibilities_list.append(visibilities)
 
@@ -420,7 +426,7 @@ class OperateImageGalaxies(OperateImageList):
         return galaxy_blurred_image_2d_dict
 
     def galaxy_visibilities_dict_from(
-        self, grid, transformer
+        self, grid, transformer, xp=np
     ) -> Dict[Galaxy, aa.Visibilities]:
         """
         Evaluate the light object's dictionary mapping galaixes to their corresponding 2D images and transform each
@@ -453,7 +459,7 @@ class OperateImageGalaxies(OperateImageList):
         for galaxy_key in galaxy_image_2d_dict.keys():
             image_2d = galaxy_image_2d_dict[galaxy_key]
 
-            if not jnp.any(image_2d.array):
+            if not xp.any(image_2d.array):
                 visibilities = aa.Visibilities.zeros(
                     shape_slim=(transformer.uv_wavelengths.shape[0],)
                 )
