@@ -29,7 +29,9 @@ class MockLightProfile(ag.LightProfile):
 
     @aa.grid_dec.to_array
     @check_operated_only
-    def image_2d_from(self, grid, operated_only: Optional[bool] = None, **kwargs):
+    def image_2d_from(
+        self, grid, xp=np, operated_only: Optional[bool] = None, **kwargs
+    ):
         if self.image_2d is not None:
             return self.image_2d
 
