@@ -1,3 +1,5 @@
+import numpy as np
+
 import autogalaxy as ag
 
 
@@ -19,11 +21,11 @@ class MockMassProfile(ag.mp.MassProfile):
         self.value = value
         self.value1 = value1
 
-    def convergence_2d_from(self, grid):
+    def convergence_2d_from(self, grid, xp=np):
         return self.convergence_2d
 
-    def potential_2d_from(self, grid):
+    def potential_2d_from(self, grid, xp=np):
         return self.potential_2d
 
-    def deflections_yx_2d_from(self, grid):
+    def deflections_yx_2d_from(self, grid, xp=np):
         return self.deflections_2d
