@@ -1,5 +1,3 @@
-import jax
-import jax.numpy as jnp
 import numpy as np
 
 from typing import List, Tuple
@@ -122,8 +120,8 @@ class AbstractSersic(MassProfile, MassProfileMGE, MassProfileCSE, StellarProfile
         self.effective_radius = effective_radius
         self.sersic_index = sersic_index
 
-    def deflections_yx_2d_from(self, grid: aa.type.Grid2DLike, **kwargs):
-        return self.deflections_2d_via_cse_from(grid=grid, **kwargs)
+    def deflections_yx_2d_from(self, grid: aa.type.Grid2DLike, xp=np, **kwargs):
+        return self.deflections_2d_via_cse_from(grid=grid, xp=xp, **kwargs)
 
     @aa.grid_dec.to_vector_yx
     @aa.grid_dec.transform
@@ -145,14 +143,14 @@ class AbstractSersic(MassProfile, MassProfileMGE, MassProfileCSE, StellarProfile
         """
         return self._deflections_2d_via_mge_from(
             grid=grid,
-            sigmas_factor=np.sqrt(self.axis_ratio),
+            sigmas_factor=np.sqrt(self.axis_ratio()),
             func_terms=func_terms,
             func_gaussians=func_gaussians,
         )
 
     @aa.grid_dec.to_vector_yx
     @aa.grid_dec.transform
-    def deflections_2d_via_cse_from(self, grid: aa.type.Grid2DLike, **kwargs):
+    def deflections_2d_via_cse_from(self, grid: aa.type.Grid2DLike, xp=np, **kwargs):
         """
         Calculate the projected 2D deflection angles from a grid of (y,x) arc second coordinates, by computing and
         summing the convergence of each individual cse used to decompose the mass profile.
@@ -166,12 +164,12 @@ class AbstractSersic(MassProfile, MassProfileMGE, MassProfileCSE, StellarProfile
         grid
             The grid of (y,x) arc-second coordinates the convergence is computed on.
         """
-        return self._deflections_2d_via_cse_from(grid=grid, **kwargs)
+        return self._deflections_2d_via_cse_from(grid=grid, xp=xp, **kwargs)
 
     @aa.over_sample
     @aa.grid_dec.to_array
     @aa.grid_dec.transform
-    def convergence_2d_from(self, grid: aa.type.Grid2DLike, **kwargs):
+    def convergence_2d_from(self, grid: aa.type.Grid2DLike, xp=np, **kwargs):
         """Calculate the projected convergence at a given set of arc-second gridded coordinates.
 
         Parameters
@@ -181,14 +179,19 @@ class AbstractSersic(MassProfile, MassProfileMGE, MassProfileCSE, StellarProfile
 
         """
         return self.convergence_func(
-            self.eccentric_radii_grid_from(grid=grid, **kwargs)
+            self.eccentric_radii_grid_from(grid=grid, xp=xp, **kwargs)
         )
 
     @aa.over_sample
     @aa.grid_dec.to_array
     @aa.grid_dec.transform
     def convergence_2d_via_mge_from(
-        self, grid: aa.type.Grid2DLike, func_terms=28, func_gaussians=20, **kwargs
+        self,
+        grid: aa.type.Grid2DLike,
+        xp=np,
+        func_terms=28,
+        func_gaussians=20,
+        **kwargs,
     ):
         """
         Calculate the projected convergence at a given set of arc-second gridded coordinates.
@@ -200,7 +203,7 @@ class AbstractSersic(MassProfile, MassProfileMGE, MassProfileCSE, StellarProfile
 
         """
 
-        eccentric_radii = self.eccentric_radii_grid_from(grid=grid, **kwargs)
+        eccentric_radii = self.eccentric_radii_grid_from(grid=grid, xp=xp, **kwargs)
 
         return self._convergence_2d_via_mge_from(
             grid_radii=eccentric_radii,
@@ -211,7 +214,7 @@ class AbstractSersic(MassProfile, MassProfileMGE, MassProfileCSE, StellarProfile
     @aa.over_sample
     @aa.grid_dec.to_array
     @aa.grid_dec.transform
-    def convergence_2d_via_cse_from(self, grid: aa.type.Grid2DLike, **kwargs):
+    def convergence_2d_via_cse_from(self, grid: aa.type.Grid2DLike, xp=np, **kwargs):
         """
         Calculate the projected 2D convergence from a grid of (y,x) arc second coordinates, by computing and summing
         the convergence of each individual cse used to decompose the mass profile.
@@ -226,7 +229,7 @@ class AbstractSersic(MassProfile, MassProfileMGE, MassProfileCSE, StellarProfile
             The grid of (y,x) arc-second coordinates the convergence is computed on.
         """
 
-        elliptical_radii = self.elliptical_radii_grid_from(grid=grid, **kwargs)
+        elliptical_radii = self.elliptical_radii_grid_from(grid=grid, xp=xp, **kwargs)
 
         return self._convergence_2d_via_cse_from(grid_radii=elliptical_radii)
 
@@ -234,7 +237,7 @@ class AbstractSersic(MassProfile, MassProfileMGE, MassProfileCSE, StellarProfile
         return self.mass_to_light_ratio * self.image_2d_via_radii_from(grid_radius)
 
     @aa.grid_dec.to_array
-    def potential_2d_from(self, grid: aa.type.Grid2DLike, **kwargs):
+    def potential_2d_from(self, grid: aa.type.Grid2DLike, xp=np, **kwargs):
         return np.zeros(shape=grid.shape[0])
 
     def image_2d_via_radii_from(self, radius: np.ndarray):
@@ -313,7 +316,7 @@ class AbstractSersic(MassProfile, MassProfileMGE, MassProfileCSE, StellarProfile
             mass_to_light_gradient=0.0,
         )
 
-        scaled_effective_radius = self.effective_radius / np.sqrt(self.axis_ratio)
+        scaled_effective_radius = self.effective_radius / np.sqrt(self.axis_ratio())
         radii_min = scaled_effective_radius / 10.0**lower_dex
         radii_max = scaled_effective_radius * 10.0**upper_dex
 
@@ -354,7 +357,7 @@ class AbstractSersic(MassProfile, MassProfileMGE, MassProfileCSE, StellarProfile
 
     @property
     def ellipticity_rescale(self):
-        return 1.0 - ((1.0 - self.axis_ratio) / 2.0)
+        return 1.0 - ((1.0 - self.axis_ratio()) / 2.0)
 
     @property
     def elliptical_effective_radius(self):
@@ -366,13 +369,15 @@ class AbstractSersic(MassProfile, MassProfileMGE, MassProfileCSE, StellarProfile
         The elliptical effective radius instead describes the major-axis radius of the ellipse containing \
         half the light, and may be more appropriate for highly flattened systems like disk galaxies.
         """
-        return self.effective_radius / np.sqrt(self.axis_ratio)
+        return self.effective_radius / np.sqrt(self.axis_ratio())
 
 
 class Sersic(AbstractSersic, MassProfileMGE, MassProfileCSE):
     @aa.grid_dec.to_vector_yx
     @aa.grid_dec.transform
-    def deflections_2d_via_integral_from(self, grid: aa.type.Grid2DLike, **kwargs):
+    def deflections_2d_via_integral_from(
+        self, grid: aa.type.Grid2DLike, xp=np, **kwargs
+    ):
         """
         Calculate the deflection angles at a given set of arc-second gridded coordinates.
 
@@ -387,10 +392,10 @@ class Sersic(AbstractSersic, MassProfileMGE, MassProfileCSE):
         def calculate_deflection_component(npow, index):
             sersic_constant = self.sersic_constant
 
-            deflection_grid = self.axis_ratio * grid.array[:, index]
+            deflection_grid = self.axis_ratio() * grid.array[:, index]
 
             for i in range(grid.shape[0]):
-                deflection_grid = deflection_grid.at[i].multiply(
+                deflection_grid[i] = deflection_grid[i] * (
                     self.intensity
                     * self.mass_to_light_ratio
                     * quad(
@@ -401,7 +406,7 @@ class Sersic(AbstractSersic, MassProfileMGE, MassProfileCSE):
                             grid.array[i, 0],
                             grid.array[i, 1],
                             npow,
-                            self.axis_ratio,
+                            self.axis_ratio(),
                             self.sersic_index,
                             self.effective_radius,
                             sersic_constant,
@@ -415,7 +420,7 @@ class Sersic(AbstractSersic, MassProfileMGE, MassProfileCSE):
         deflection_x = calculate_deflection_component(0.0, 1)
 
         return self.rotated_grid_from_reference_frame_from(
-            np.multiply(1.0, np.vstack((deflection_y, deflection_x)).T)
+            np.multiply(1.0, np.vstack((deflection_y, deflection_x)).T), xp=xp
         )
 
     @staticmethod
